'''
ultipa 常量设置
'''

LIMIT = 1

DEPTH = 1

GETLEADER_TIMEOUT = 5
# dateTime 类型是否转为时间格式的字符串  1970-01-01 08:00:00
DateTimeIs2Str = True
# Timestamp 类型是否转为时间格式的字符串  1970-01-01 08:00:00 （根据当前时区进行转换）
TimestampIs2Str = False
# 精度控制返回7位
Float_Accuracy = True