# -*- coding: utf-8 -*-
# @Time    : 2023/7/18 11:53
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : clientType.py
class ClientType:
	Default = 0  # 默认
	Algo = 1  # 算法
	Update = 2  # 更新
	Leader = 3  # 主节点
